<?php
if ( ! defined('EXPRTZIN_BITPAY_LLMS_VERSION')) exit;


if( ! class_exists( 'ExprtzIN_Bitpay_Llms_Gateway' ) ) :
	class ExprtzIN_Bitpay_Llms_Gateway extends LLMS_Payment_Gateway {

	    /**
	     * @var string
	     * @since 3.0.0
	     */
	    public $token;

	    /**
	     * Constructor
	     *
	     * @return  void
	     * @since   3.0.0
	     * @version 3.10.0
	     */
	    public function __construct() {

	        $this->id                    = 'exprtzin_bitpay';
	        $this->admin_description     = __( 'Collect Crypto payments using Bitpay', 'expert-bitpay-gateway-lifterlms' );
	        $this->admin_title           = 'Bitpay';
	        $this->title                 = 'Bitpay';
	        $this->description           = __( 'Pay with BitPay using one of the supported cryptocurrencies', 'expert-bitpay-gateway-lifterlms' );
	        $this->test_mode_title       = __( 'Testnet', 'expert-bitpay-gateway-lifterlms' );
	        $this->test_mode_description = '<span class="description">'.__( 'Bitpay testnet can be used to test payments.', 'expert-bitpay-gateway-lifterlms' ).'</span>';
	        $this->token                 = '';

	        $this->supports = array(
	            'checkout_fields'    => true,
	            'single_payments'    => true,
	            'test_mode'          => true,
	        );

	        add_filter( 'llms_get_gateway_settings_fields', array( $this, 'get_settings_fields' ), 10, 2 );

	    }

	    /**
	     * Get admin setting fields
	     *
	     * @param    array  $fields      default fields
	     * @param    string $gateway_id  gateway ID
	     * @return   array
	     * @since    3.0.0
	     * @version  3.0.0
	     */
	    public function get_settings_fields( $fields, $gateway_id ) {

	        if ( $this->id !== $gateway_id ) {
	            return $fields;
	        }

	        $fields[] = array(
	            'id'    => $this->get_option_name( 'token' ),
	            'desc'  => '<br>' . __( 'Your <b>development</b> API token.  <a href = "https://test.bitpay.com/dashboard/merchant/api-tokens" target = "_blank">Create one here</a> and <b>uncheck</b> `Require Authentication`.', 'expert-bitpay-gateway-lifterlms' ),
	            'title' => __( 'Merchant API token', 'expert-bitpay-gateway-lifterlms' ),
	            'type'  => 'password',
	        );

	        return $fields;

	    }

	    /**
	     * Handle a Pending Order
	     * Called by LLMS_Controller_Orders->create_pending_order() on checkout form submission
	     * All data will be validated before it's passed to this function
	     *
	     * @param   obj       $order   Instance LLMS_Order for the order being processed
	     * @param   obj       $plan    Instance LLMS_Access_Plan for the order being processed
	     * @param   obj       $person  Instance of LLMS_Student for the purchasing customer
	     * @param   obj|false $coupon  Instance of LLMS_Coupon applied to the order being processed, or false when none is being used
	     * @return  void
	     * @since   3.0.0
	     * @version 3.10.0
	     */
	    public function handle_pending_order( $order, $plan, $person, $coupon = false ) {

	        if ( $order->get_price( 'total', array(), 'float' ) > 0  ) {

	        	$product = $plan->get_product();

	        	$orderData = [
				    'currency' => get_lifterlms_currency(),
				    'price' => $order->get('total'),
				    'orderId' => $order->get('id'),
				    'notificationURL' => site_url() . '/?action=bitpay_notification',
				    'transactionSpeed' => 'high', //high, medium, low
				    'redirectURL' => $order->get_view_link(),
				    'buyer' => [
				         'email' => $order->get('billing_email')
				    ],
				    'token' => $this->get_option('token')
				];

		        $response = $this->create_invoice( $orderData );

			    if ( empty( $response['error'] ) ) {

			    	$order->add_note( __( 'Waiting for the payment.', 'expert-bitpay-gateway-lifterlms' ) );
            		llms_add_notice( __( 'Waiting for the payment.', 'expert-bitpay-gateway-lifterlms' ), 'notice' );

            		if ( ! empty( $response['data'] ) && ! empty( $response['data']->url ) ) {

			    		$order->add_note( __( 'Invoice ID: ('. $response['data']->id .').', 'expert-bitpay-gateway-lifterlms' ) );

			    		update_post_meta( $order->get('id'), 'bitpay_invoice_id', $response['data']->id );

		        		llms_redirect_and_exit( 
		        			$response['data']->url, 
		        			array( 'safe' => false ) 
		        		);        	

            		}

	        		llms_redirect_and_exit( $order->get_view_link() );

			    }
			    else {

			    	$errors = (string) $response['error'];

			    	$order->add_note( $errors );
        			llms_add_notice( $errors, 'error' );
			    	
	        		llms_redirect_and_exit( $order->get_view_link() );

			    }
                
	        }

	    }

	    /**
	     * Determine if the gateway is enabled according to admin settings checkbox
	     *
	     * @return   boolean
	     * @since    3.0.0
	     * @version  3.0.0
	     */
	    public function is_enabled() {
	        return ( 'yes' === $this->get_enabled() ) ? true : false;
	    }

	    private function create_invoice( $request ) {

	    	$host = 'https://bitpay.com/';

	        if ( $this->get_option('test_mode_enabled') == 'yes' )
		        $host = 'https://test.bitpay.com/';

	    	$response = wp_remote_post( $host . 'invoices', array(
				    'method'      => 'POST',
				    'headers'     => array( 
				    	'X-Accept-Version' => '2.0.0', 
				    	'Content-Type' => 'application/json' 
				    ),
				    'sslverify'   => false,
				    'body'        => stripslashes( json_encode( $request ) )
				)
	    	);

			if ( ! empty( $response ) && ! is_wp_error( $response ) ) {
	        
	            $response = json_decode( $response['body'] );
	        
	            if ( isset( $response->data ) ) {
	                
	                return array( 'data' => $response->data );
	       
	            }
	            elseif ( isset( $response->error ) ) {

	            	$error = is_string( $response->error ) ? $response->error : json_encode( $response->error );
	       
	                return array( 'error' => $error );
	       
	            }
	            else {

	            	return array( 'error' => "Something went wrong." );
	            
	            }
	        
	        } 
	        elseif ( is_wp_error( $response ) ) {

			    return array( 'error' => "Something went wrong." );
			
			} 
			else {

			    return array( 'error' => "Something went wrong." );
			
			}

	    }
	}
endif;

